/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.action;

import javax.annotation.Resource;

import net.sourceforge.kivu4j.utils.json.JSON;
import net.sourceforge.kivu4j.utils.json.JsonUtil;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * 客户端请求入口，Action父类，所有的Action继承此类。
 * 
 * <p>
 * 这个类只处理json格式的数据，接受json请求数据，返回json结果数据。
 * </p>
 * 
 * <p>
 * 当接收json请求数据时，请求数据可以是字符串，数值，map，以及领域对象，此类将请求数据转换成java对象。
 * 这个类提供了很多转换函数，用来处理各种类型的json数据。处理完后，需要返回给客户端结果数据。结果数据同样
 * 可以是字符串，数值，map，对象等等.
 * </p>
 * 
 * <p>
 * 分页的实现。出于性能及必要性的考虑，不统计总页数，只提示是否有下一页。后台执行查询时，设置记录条数是
 * limit+1，如果返回了全部数据，就存在下一页；反之，就没有下一页。
 * </p>
 * 
 * @author Lucky Yang
 */
public class JSONActionSupport{
	public static final String SUCCESS="success";
	
	protected Logger logger = LoggerFactory.getLogger(this.getClass());
	
	@Resource
	protected JsonUtil jsonUtil;
	
	private Long sessionId;
	private int start;
	private int limit;	
	private int total;
	private String actionName;
	private String namespace;
	private String requestData;
	private ResponseData responseData;
	
	/**
	 * 在请求的json数据外，加一个root节点。
	 * 
	 * @return string
	 */
	private String addRootNode(){
		String rslt;
		if (StringUtils.isBlank(this.requestData))
			rslt = JSON.EMPTY_JSON;
		else
			rslt = "{root:" + this.requestData + "}";	
		
		return rslt;
	}
	
	/**
	 * 请求数据转换成对象
	 * 
	 * @param <T>
	 * @param request 字符串
	 * @param clazz 特定的类
	 * @return
	 * @throws ActionRequestDataIsBlankException 如果字符串为空或<code>null</code>
	 */
	private <T> T getRequest(String request, Class<T> clazz){
		if (StringUtils.isBlank(request))
			throw new ActionRequestDataIsBlankException();
		
		if (logger.isDebugEnabled()){
			logger.debug("[Request JSON]:" + request);
		}
		return this.jsonUtil.fromJson(request, clazz);		
	}
	
	/**
	 * 请求数据转换成对象
	 * 
	 * @param <T> class
	 * @param clazz class
	 * @return the instance of clazz
	 * @throws ActionRequestDataIsBlankException 如果<code>requestData</code>为空或<code>null</code>
	 */
	protected <T> T getRequest(Class<T> clazz){
		return this.getRequest(this.requestData, clazz);
	}

	/**
	 * 将请邱数据转换成<code>RequestFilter</code>实例
	 * 
	 * @return 
	 */
	protected RequestFilter getRequestAsFilter(){
		String request = this.addRootNode();
		
		return this.getRequest(request, RequestFilter.class);
	}
	
	/**
	 * 将请邱数据转换成<code>RequestMap</code>实例
	 * 
	 * @return the instance of <code>getRequestMap</code> class
	 */	
	protected RequestMap getRequestAsMap(){
		String request = this.addRootNode();
		
		return this.getRequest(request, RequestMap.class);	
	}
	
	/**
	 * 将请邱数据转换成<code>String</code>实例
	 * 
	 * @return
	 */
	protected String getRequestAsStr(){
		return this.getRequest(this.requestData, String.class);	
	}	
	
	/**
	 * 将请邱数据转换成<code>Long</code>实例
	 * 
	 * @return
	 */
	protected Long getRequestAsLong(){
		return this.getRequest(this.requestData, Long.class);	
	}
	
	/**
	 * 处理异常，记录错误到日志，并将错误返回给客户端
	 * 
	 * @param t 异常
	 */
	protected void handleException(Throwable t){
		this.responseData = new ResponseError(t);
		
		logger.error("ERROR", t);
	}	

	public String getRequestData() {
		return requestData;
	}

	public void setRequestData(String requestData) {	
		this.requestData = requestData;
	}

	public ResponseData getResponseData() {
		return responseData;
	}

	/**
	 * 对结果数据分页
	 * 
	 * @param responseData
	 */
	public void setResponseData(ResponseData responseData) {
		if (responseData != null){
			responseData.setStart(start);
			responseData.setLimit(limit);
			responseData.setTotal(total);
			responseData.doPagination();
		}
		
		if (logger.isDebugEnabled()){
			logger.debug("[Response JSON]:" + responseData);
		}			
		this.responseData = responseData;
	}

	public String getActionName() {
		return actionName;
	}

	public void setActionName(String actionName) {
		this.actionName = actionName;
	}

	public String getNamespace() {
		return namespace;
	}

	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public int getStart() {
		return start;
	}

	public void setStart(int start) {
		this.start = start;
	}

	public int getLimit() {
		return limit;
	}

	public void setLimit(int limit) {
		this.limit = limit;
	}

	public int getTotal() {
		return total;
	}

	public void setTotal(int total) {
		this.total = total;
	}

	public Long getSessionId() {
		return sessionId;
	}

	public void setSessionId(Long sessionId) {
		this.sessionId = sessionId;
	}


	
	
}
